/*
	ad.rangeslider
	a jsui rangeslider - version 5
	anders.dahl@underhund.com	www.underhund.com/anders/
*/

// autowatch = 1;

// -------------------------------
// Set up inlets/outlets/assist strings
inlets = 2;
outlets = 2;
setinletassist(0, "Low range limit (float), Messages");
setinletassist(1, "High range limit (float)");
setoutletassist(0, "Low range limit (float)");
setoutletassist(1, "High range limit (float)");

// -------------------------------
// Global varables and code

sketch.default2d();
var lo = 0.;
var lorange = 0.;
var button = 0.;
var drag = 0.;
var orientation = 1; // 0 = horizontal, 1 = vertical
var hi = 1;
var hirange = 1;
var totalrange = 1.;
var lastv = 0.5;
var vbrgb = [1., 1., 1., 1.];
var vfrgb = [0.4705, 0.4705, 0.4705, 1.]; // 120 120 120
sketch.fsaa = 0;

var aspect, width, height, wpixel, hpixel, hpane, wpane;

// -------------------------------
// Process arguments

// range
if (jsarguments.length>1)
	lorange = jsarguments[1];
if (jsarguments.length>2)
	hirange = jsarguments[2];

// frgb
if (jsarguments.length>3)
	vfrgb[0] = jsarguments[3]/255.;
if (jsarguments.length>4)
	vfrgb[1] = jsarguments[4]/255.;
if (jsarguments.length>5)
	vfrgb[2] = jsarguments[5]/255.;

// brgb
if (jsarguments.length>6)
	vbrgb[0] = jsarguments[6]/255.;
if (jsarguments.length>7)
	vbrgb[1] = jsarguments[7]/255.;
if (jsarguments.length>8)
	vbrgb[2] = jsarguments[8]/255.;

notifyclients();
onresize();
range(lorange, hirange, 1);
draw();

// -------------------------------
// Inlet functions

function bang()
{
	var loout, hiout;
	draw();
	refresh();
	if (lo<hi)
	{
		loout = denormalize(clip(lo));
		hiout = denormalize(clip(hi));
	}
	else
	{
		loout = denormalize(clip(hi));
		hiout = denormalize(clip(lo));
	}
	
	outlet(0, loout);
	outlet(1, hiout);
}

function msg_float(v)
{
	v = normalize(v);
	if (inlet == 0)
	{
		lo = v;
		notifyclients();
		bang();
	}
	else
	{
		hi = v;
		notifyclients();
	}
}

function list(a, b)
{
	lo = normalize(a);
	hi = normalize(b);
	
	notifyclients();
	bang();
}

function set(a, b)
{
	lo = normalize(a);
	hi = normalize(b);
	
	notifyclients();
	draw();
	refresh();
}

function range(a, b, mute)
{
	lorange = a;
	hirange = b;
	totalrange = hirange - lorange;
	if (!mute)
		bang();
}

// -------------------------------
// Mouse functions

function onclick(x,y,but,cmd,shift,capslock,option,ctrl)
{
	ondrag(x,y,but,cmd,shift,capslock,option,ctrl)
}
onclick.local = 1; //private. could be left public to permit "synthetic" events

function ondblclick(x,y,but,cmd,shift,capslock,option,ctrl)
{
	lo = 0.;
	hi = 1.;
	notifyclients();
	bang();
	// ondrag(x,y,but,cmd,shift,capslock,option,ctrl) // Why doesn't this work?
}
ondblclick.local = 1; //private.

function ondrag(x,y,but,cmd,shift,capslock,option,ctrl)
{
	var f,a;
	var newdrag = 0;
	
	if (but==1 && button==0)
		newdrag = 1;
	
	button = but;
	a = sketch.screentoworld(x,y);
	f = (a[orientation]+1)*0.5; // 0-1
	if (orientation == 0)
		f = clip((a[orientation]/aspect+1)*0.5); // 0-1
	else
		f = clip((a[orientation]+1)*0.5); // 0-1
	mod = cmd + ctrl;
	changerange(f, newdrag, mod, shift, option);
}
ondrag.local = 1; //private.

function onresize(w,h)
{
	width = sketch.size[0];
	height = sketch.size[1];
	wpixel = 2/width;
	hpixel = 2/height;
	wpane = 1-wpixel*2;
	hpane = 1-hpixel*2;
	
	aspect = width/height;
	if (width<height)
		orientation = 1;
	else
		orientation = 0;
	
	draw();
	refresh();
}
onresize.local = 1; //private

// -------------------------------
// Draw

function draw()
{
	with (sketch)
	{
		//scale everything to box size
		glmatrixmode("modelview");
		glpushmatrix();
		glscale(aspect,1,1);

		// erase background
		glclearcolor(vbrgb[0],vbrgb[1],vbrgb[2],vbrgb[3]);
		glclear();

		// draw range plane
		glcolor(vfrgb);
		shapeslice(1,1);
		if (orientation == 1)
		{ // vertical
			moveto(0, (lo+hi)-1); // center point. on screen in range -1. to 1
			plane(wpane, Math.max(Math.abs(hi-lo), hpixel)); // range rect, minimum size 1px  
		}
		else
		{ // horizontal
			moveto((lo+hi)-1, 0); // center point. on screen in range -1. to 1
			plane(Math.max(Math.abs(hi-lo), wpixel), hpane); // range rect, minimum size 1px
		}
		//reset transformation matrix
		glmatrixmode("modelview");
		glpopmatrix();
	}
}

// -------------------------------
// Functions

function normalize(v)
{
	v = clip((v-lorange)/totalrange);
	return v;
}
normalize.local = 1; //private

function denormalize(v)
{
	v = v*totalrange+lorange;
	return v;
}
denormalize.local = 1; //private

function clip(v)
{
	v = Math.min(Math.max(0,v),1);
	return v;
}
clip.local = 1; //private

function changerange(v, newdrag, mod, shift, option)
{
	var mean, sixt;
	if (newdrag)
	{
		mean = (hi+lo)*0.5;
		startlo= lo;
		starthi = hi;
		lastv = v;
		
		if (mod) // Range drag
			drag = 2;
		else
		{
			if (shift)  // Range inc/dec
				drag = 3;
			else
			{
				sixt = Math.abs(hi-lo)/6.;
				if (v > mean-sixt && v < mean+sixt && option == 0)
					drag = 2; // drag the middle third of the slider
				else
				{
					if (Math.abs(lo-v) < Math.abs(hi-v))
						drag = 0; // drag the lo third of the slider
					else
						drag = 1; // drag the hi third of the slider
				}
			}
		}
	}
	
	switch (drag)
	{
		case 0 : // Lo drag
			lo = v;
			break;
		case 1 : // Hi drag
			hi = v;
			break;
		case 2 : // Range drag
			lo = lo+v-lastv;
			hi = hi+v-lastv;
			lastv = v;
			break;
		case 3 : // Range inc/dec
			lo = startlo-(v-lastv);
			hi = starthi+(v-lastv);
			break;
		default :
			post("Warning: Drag type error\n");
	}

	notifyclients();
	bang();
}
changerange.local = 1; //private

// -------------------------------
// Pattr

function setvalueof(a,b)
{
	lo = a;
	hi = b;
	notifyclients();
	bang();
}

function getvalueof()
{
	var value = new Array;
	if (lo<hi)
	{
		value[0] = clip(lo);
		value[1] = clip(hi);
	}
	else
	{
		value[0] = clip(hi);
		value[1] = clip(lo);
	}
	return value;
}

// -------------------------------
// Graphic

function fsaa(v)
{
	sketch.fsaa = v;
	bang();
}

function frgb(r,g,b)
{
	vfrgb[0] = r/255.;
	vfrgb[1] = g/255.;
	vfrgb[2] = b/255.;
	draw();
	refresh();
}

function brgb(r,g,b)
{
	vbrgb[0] = r/255.;
	vbrgb[1] = g/255.;
	vbrgb[2] = b/255.;
	draw();
	refresh();
}

/*
	TODO:
	1. An option to draw ticks (black & white w. 50% opacity)
	2. Move function ( same as range drag )
*/
